@@ -94,7 +94,7 @@ SetupUI::SetupUI (const ConfigPointer &config, const String &display, const Help
     create_main_ui ();
     create_module_list_model ();
 
-    m_query_changed_timeout = gtk_timeout_add (200, query_changed_timeout_cb, this);
+    m_query_changed_timeout = g_timeout_add (200, query_changed_timeout_cb, this);
 
     m_helper_agent.open_connection (helper_info, display);
 
@@ -252,7 +252,11 @@ SetupUI::create_main_ui ()
 
     // Create statusbar.
     m_status_bar = gtk_statusbar_new ();
+#if GTK_CHECK_VERSION(3, 0, 0)
+    gtk_window_set_has_resize_grip (GTK_WINDOW(m_main_window), TRUE);
+#else
     gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (m_status_bar), TRUE);
+#endif
     gtk_widget_show (m_status_bar);
     gtk_box_pack_start (GTK_BOX (vbox1), m_status_bar, FALSE, FALSE, 0);
 
@@ -306,7 +310,7 @@ SetupUI::create_main_ui ()
     gtk_widget_show (ok_button);
     gtk_box_pack_end (GTK_BOX (hbox1), ok_button, FALSE, FALSE, 4);
 
-    exit_button = gtk_button_new_from_stock ("gtk-quit");
+    exit_button = gtk_button_new_from_stock ("gtk-cancel");
     gtk_widget_show (exit_button);
     gtk_box_pack_end (GTK_BOX (hbox1), exit_button, FALSE, FALSE, 4);
 
@@ -317,7 +321,11 @@ SetupUI::create_main_ui ()
     m_apply_button = gtk_button_new_from_stock ("gtk-apply");
     gtk_widget_show (m_apply_button);
     gtk_box_pack_end (GTK_BOX (hbox1), m_apply_button, FALSE, FALSE, 4);
+#if GTK_CHECK_VERSION(2, 18, 0)
+    gtk_widget_set_can_default (m_apply_button, TRUE);
+#else
     GTK_WIDGET_SET_FLAGS (m_apply_button, GTK_CAN_DEFAULT);
+#endif
     gtk_widget_set_sensitive (m_apply_button, FALSE);
 
     m_restore_button = gtk_button_new_from_stock ("gtk-revert-to-saved");
@@ -640,12 +648,21 @@ SetupUI::query_changed_timeout_cb (gpointer data)
         ui->m_current_module && ui->m_current_module->query_changed ())
         modified = true;
 
+#if GTK_CHECK_VERSION(2, 18, 0)
+    if (gtk_widget_get_sensitive (ui->m_apply_button) != modified)
+#else
     if (GTK_WIDGET_SENSITIVE (ui->m_apply_button) != modified)
+#endif
         gtk_widget_set_sensitive (ui->m_apply_button, modified);
 
+#if GTK_CHECK_VERSION(2, 18, 0)
+    if (gtk_widget_get_sensitive (ui->m_restore_button) != modified)
+#else
     if (GTK_WIDGET_SENSITIVE (ui->m_restore_button) != modified)
+#endif
         gtk_widget_set_sensitive (ui->m_restore_button, modified);
 
+
     return TRUE;
 }
 
