@@ -1,7 +1,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_module.cpp,v 1.22 2004/07/20 10:33:53 suzhe Exp $
+ * $Id: scim_module.cpp,v 1.26 2005/05/24 12:22:51 suzhe Exp $
  *
  */
 
@@ -27,7 +27,9 @@
 #define Uses_STL_ALGORITHM
 #include "scim_private.h"
 #include "scim.h"
-#include "ltdl.h"
+extern "C" {
+  #include <ltdl.h>
+}
 #include <dirent.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -158,7 +160,8 @@ Module::load (const String &name, const String &type)
 
     String module_path;
 
-    lt_dlhandle    new_handle;
+    lt_dlhandle    new_handle = 0;
+
     ModuleInitFunc new_init;
     ModuleExitFunc new_exit;
 
@@ -173,7 +176,6 @@ Module::load (const String &name, const String &type)
 
     if (!new_handle) {
         new_handle = lt_dlopenext (name.c_str ());
-        module_path = String ();
     }
 
     if (!new_handle)
@@ -231,10 +233,12 @@ Module::load (const String &name, const String &type)
     if (unload ()) {
         _scim_modules.push_back (new_init);
 
+        const lt_dlinfo *info = lt_dlgetinfo (new_handle);
+
         m_impl->handle = new_handle;
         m_impl->init   = new_init;
         m_impl->exit   = new_exit;
-        m_impl->path   = module_path;
+        m_impl->path   = String (info->filename);
         m_impl->name   = name;
 
         try {
