@@ -547,6 +547,11 @@ scim_get_home_dir ()
 
     struct passwd *pw;
 
+    home_dir = getenv ("SCIM_HOME");
+    if (home_dir && *home_dir) {
+        return String (home_dir);
+    }
+
     setpwent ();
     pw = getpwuid (getuid ());
     endpwent ();
@@ -568,6 +573,11 @@ scim_get_user_name ()
     struct passwd *pw;
     const char *user_name;
 
+    user_name = getenv ("SCIM_USER");
+    if (user_name && *user_name) {
+        return String (user_name);
+    }
+
     setpwent ();
     pw = getpwuid (getuid ());
     endpwent ();
@@ -766,6 +776,8 @@ static __Language __languages [] = {
     { "pt_PT",    NULL, N_("Portuguese"), "Português", "@euro" },
     { "ro_RO",    NULL, N_("Romanian"), "Română", NULL },
     { "ru_RU",    NULL, N_("Russian"), "русский", ".koi8r" },
+    { "sd",    "sd_IN", N_("Sindhi"), "ﺲﻧڌﻱ", NULL },
+    { "sd_IN",    NULL, N_("Sindhi"), "सिन्धी", "@devanagari" },
     { "si_LK",    NULL, N_("Sinhala"), "සිංහල", NULL },
     { "sk_SK",    NULL, N_("Slovak"), "Slovenský", NULL },
     { "sl_SI",    NULL, N_("Slovenian"), "Slovenščina", NULL },
