@@ -111,18 +111,25 @@ static void
 scim_key_selection_class_init (ScimKeySelectionClass *klass)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+#if GTK_CHECK_VERSION(3, 0, 0)
+#else
     GtkObjectClass *object_class;
     GtkWidgetClass *widget_class;
 
     object_class = (GtkObjectClass*) klass;
     widget_class = (GtkWidgetClass*) klass;
+#endif
     parent_class = (GtkWidgetClass*) g_type_class_peek_parent (klass);
 
     gobject_class->finalize = scim_key_selection_finalize;
 
     key_selection_signals[KEY_SELECTION_CHANGED] =
         g_signal_new ("key-selection-changed",
+#if GTK_CHECK_VERSION(3, 0, 0)
+                    G_TYPE_FROM_CLASS (gobject_class),
+#else
                     G_TYPE_FROM_CLASS (object_class),
+#endif
                     G_SIGNAL_RUN_FIRST,
                     G_STRUCT_OFFSET (ScimKeySelectionClass, changed),
                     NULL,
@@ -213,7 +220,7 @@ scim_key_selection_init (ScimKeySelection *keyselection)
     keyselection->key_code = gtk_entry_new ();
     gtk_widget_show (keyselection->key_code);
     gtk_box_pack_start (GTK_BOX (hbox), keyselection->key_code, TRUE, TRUE, 2);
-    gtk_entry_set_editable (GTK_ENTRY (keyselection->key_code), FALSE);
+    gtk_editable_set_editable (GTK_EDITABLE (keyselection->key_code), FALSE);
 
     button = gtk_button_new_with_label (_("..."));
     gtk_widget_show (button);
@@ -267,6 +274,11 @@ scim_key_selection_init (ScimKeySelection *keyselection)
     gtk_widget_show (keyselection->toggle_hyper);
     gtk_box_pack_start (GTK_BOX (hbox), keyselection->toggle_hyper, TRUE, TRUE, 2);
 
+
+    keyselection->toggle_capslock = gtk_check_button_new_with_mnemonic (_("Ca_psLock")); 
+    gtk_widget_show (keyselection->toggle_capslock);
+    gtk_box_pack_start (GTK_BOX (hbox), keyselection->toggle_capslock, TRUE, TRUE, 2);
+    
     hbox = gtk_hbox_new (TRUE, 0);
     gtk_widget_show (hbox);
     gtk_box_pack_start (GTK_BOX (keyselection), hbox, FALSE, FALSE, 4);
@@ -319,6 +331,8 @@ scim_key_selection_add_key_button_callback (GtkButton        *button,
         key += String ("Super+");
     if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (keyselection->toggle_hyper)))
         key += String ("Hyper+");
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (keyselection->toggle_capslock)))
+        key += String ("CapsLock+");
 
     key_code = String (gtk_entry_get_text (GTK_ENTRY (keyselection->key_code)));
     if (!key_code.length ()){
@@ -460,7 +474,7 @@ scim_key_grab_release_callback (GtkDialog   *dialog,
 GtkWidget*
 scim_key_selection_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (SCIM_TYPE_KEY_SELECTION));
+  return GTK_WIDGET (g_object_new (SCIM_TYPE_KEY_SELECTION, NULL));
 }
 
 void
@@ -571,6 +585,9 @@ scim_key_selection_set_key_event (ScimKeySelection *keyselection,
     gtk_toggle_button_set_active (
         GTK_TOGGLE_BUTTON (keyselection->toggle_hyper),
         event.is_hyper_down ());
+    gtk_toggle_button_set_active (
+        GTK_TOGGLE_BUTTON (keyselection->toggle_capslock),
+        event.is_caps_lock_down ());
     gtk_toggle_button_set_active (
         GTK_TOGGLE_BUTTON (keyselection->toggle_release),
         event.is_key_release ());
@@ -654,16 +671,28 @@ scim_key_selection_dialog_init (ScimKeySelectionDialog *keyseldialog)
     gtk_container_set_border_width (GTK_CONTAINER (keyseldialog), 4);
     gtk_window_set_resizable (GTK_WINDOW (keyseldialog), TRUE);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    keyseldialog->content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+#else
     keyseldialog->main_vbox = dialog->vbox;
+#endif
 
     keyseldialog->keysel = scim_key_selection_new ();
     gtk_container_set_border_width (GTK_CONTAINER (keyseldialog->keysel), 4);
     gtk_widget_show (keyseldialog->keysel);
+#if GTK_CHECK_VERSION(3, 0, 0)
+    gtk_box_pack_start (GTK_BOX (keyseldialog->content_area),
+#else
     gtk_box_pack_start (GTK_BOX (keyseldialog->main_vbox),
+#endif
                         keyseldialog->keysel, TRUE, TRUE, 0);
 
     /* Create the action area */
+#if GTK_CHECK_VERSION(3, 0, 0)
+    keyseldialog->action_area = gtk_dialog_get_action_area (GTK_DIALOG (dialog));
+#else
     keyseldialog->action_area = dialog->action_area;
+#endif
 
     keyseldialog->cancel_button = gtk_dialog_add_button (dialog,
                                                         GTK_STOCK_CANCEL,
@@ -677,7 +706,10 @@ scim_key_selection_dialog_init (ScimKeySelectionDialog *keyseldialog)
     gtk_window_set_title (GTK_WINDOW (keyseldialog),
                           _("Key Selection"));
 
+#if GTK_CHECK_VERSION(2, 22, 0)
+#else
     gtk_dialog_set_has_separator (GTK_DIALOG (dialog), TRUE);
+#endif
 
     gtk_widget_pop_composite_child ();
 }
