@@ -7,18 +7,18 @@
 G_BEGIN_DECLS
 
 #define SCIM_TYPE_KEY_SELECTION            (scim_key_selection_get_type ())
-#define SCIM_KEY_SELECTION(obj)            (GTK_CHECK_CAST ((obj), SCIM_TYPE_KEY_SELECTION, ScimKeySelection))
-#define SCIM_KEY_SELECTION_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), SCIM_TYPE_KEY_SELECTION, ScimKeySelectionClass))
-#define SCIM_IS_KEY_SELECTION(obj)         (GTK_CHECK_TYPE ((obj), SCIM_TYPE_KEY_SELECTION))
-#define SCIM_IS_KEY_SELECTION_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SCIM_TYPE_KEY_SELECTION))
-#define SCIM_KEY_SELECTION_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), SCIM_TYPE_KEY_SELECTION, ScimKeySelectionClass))
+#define SCIM_KEY_SELECTION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SCIM_TYPE_KEY_SELECTION, ScimKeySelection))
+#define SCIM_KEY_SELECTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SCIM_TYPE_KEY_SELECTION, ScimKeySelectionClass))
+#define SCIM_IS_KEY_SELECTION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SCIM_TYPE_KEY_SELECTION))
+#define SCIM_IS_KEY_SELECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SCIM_TYPE_KEY_SELECTION))
+#define SCIM_KEY_SELECTION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SCIM_TYPE_KEY_SELECTION, ScimKeySelectionClass))
 
 #define SCIM_TYPE_KEY_SELECTION_DIALOG            (scim_key_selection_dialog_get_type ())
-#define SCIM_KEY_SELECTION_DIALOG(obj)            (GTK_CHECK_CAST ((obj), SCIM_TYPE_KEY_SELECTION_DIALOG, ScimKeySelectionDialog))
-#define SCIM_KEY_SELECTION_DIALOG_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), SCIM_TYPE_KEY_SELECTION_DIALOG, ScimKeySelectionDialogClass))
-#define SCIM_IS_KEY_SELECTION_DIALOG(obj)         (GTK_CHECK_TYPE ((obj), SCIM_TYPE_KEY_SELECTION_DIALOG))
-#define SCIM_IS_KEY_SELECTION_DIALOG_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SCIM_TYPE_KEY_SELECTION_DIALOG))
-#define SCIM_KEY_SELECTION_DIALOG_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), SCIM_TYPE_KEY_SELECTION_DIALOG, ScimKeySelectionDialogClass))
+#define SCIM_KEY_SELECTION_DIALOG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SCIM_TYPE_KEY_SELECTION_DIALOG, ScimKeySelectionDialog))
+#define SCIM_KEY_SELECTION_DIALOG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SCIM_TYPE_KEY_SELECTION_DIALOG, ScimKeySelectionDialogClass))
+#define SCIM_IS_KEY_SELECTION_DIALOG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SCIM_TYPE_KEY_SELECTION_DIALOG))
+#define SCIM_IS_KEY_SELECTION_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SCIM_TYPE_KEY_SELECTION_DIALOG))
+#define SCIM_KEY_SELECTION_DIALOG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SCIM_TYPE_KEY_SELECTION_DIALOG, ScimKeySelectionDialogClass))
 
 typedef struct _ScimKeySelection       ScimKeySelection;
 typedef struct _ScimKeySelectionClass  ScimKeySelectionClass;
@@ -33,6 +33,10 @@ struct _ScimKeySelection
     GtkWidget *toggle_ctrl;
     GtkWidget *toggle_alt;
     GtkWidget *toggle_shift;
+    GtkWidget *toggle_meta;
+    GtkWidget *toggle_super;
+    GtkWidget *toggle_hyper;
+    GtkWidget *toggle_capslock;
     GtkWidget *toggle_release;
     GtkWidget *key_code;
 
@@ -56,7 +60,11 @@ struct _ScimKeySelectionDialog
 
     GtkWidget *keysel;
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    GtkWidget *content_area;
+#else
     GtkWidget *main_vbox;
+#endif
     GtkWidget *action_area;
 
     GtkWidget *ok_button;
